/* Lo Slave  un PIC16F88 dotato di modulo SSP
SCL --> RB1 SDA
SDA --> RB4 SCL
La comunicazione  gestita con interrupt */
#include <htc.h>
#define _XTAL_FREQ 8000000
#include "attesa16.h"

// ATTENZIONE ! Parametri di config della versione C HITECH 9.81
__CONFIG( WDTE_OFF & MCLRE_ON & LVP_OFF & FOSC_INTOSCIO);

// Indirizzo assegnato ALLO slave
#define ADDR_PIC 0x51

//==================  LED DI CONTROLLO =========================
#define ROSSOBLI 	RA6=1;attesaMs(50);RA6=0;attesaMs(50);
#define VERDEBLI 	RA7=1;attesaMs(50);RA7=0;attesaMs(50);
#define GIALLOBLI 	RB0=1;attesaMs(50);RB0=0;attesaMs(50);
#define VERDEON 	RA7=1
#define VERDEOFF 	RA7=0
#define ROSSOON 	RA6=1
#define ROSSOOFF 	RA6=0
#define GIALLOON 	RB0=1
#define GIALLOOFF 	RB0=0
// ================  VAR GLOBALI ===============================

char PARAM[4];
unsigned char COM,ind_out=0,ind_in=0;

//============== routine di Interrupt I2C ==========================
void interrupt isr(void) {
if (SSPIF && SSPIE) {
		// COM = xx(DA)(P) - (S)(RW)x(BF) 
		// COM = xx 0  x -  1   0 x 1  =9  --> !DA & S & !RW &  BF  indir scrive (9)
		// COM = xx 1  x -  1   0 x 1  =41 -->  DA & S & !RW &  BF  dato  scrive (41)
		// COM = xx 0  x -  1   1 x 0  =12 --> !DA & S &  RW & !BF  Indir per leggere e legge ? (12)
		// COM = xx 1  x -  1   1 x 0  =44 -->  DA & S &  RW & !BF  dato  legge  (44)
		// COM = xx 1  x -  0   0 x 0  =32 -->  DA &!S & !RW & !BF   (nack)     (32)
	COM=SSPSTAT & 0B00101101; 
	if (COM==9)  {	// Arriva un INDIR da Master (!DA) al fine di scrivere (!RW) dati su Slave
		 	SSPBUF=SSPBUF;				// LETTURA per AZZERARE BF
			if(SSPOV)    {SSPOV = 0;}	// Gestione EVENTUALE Overflow 
			}
	else if (COM==41) {	// Arriva un DATO da Master (DA) per essere scritto su Slave
			PARAM[ind_in++]=SSPBUF;
			if(SSPOV)   { SSPOV = 0; }	// Gestione Overflow 
			} else  if(COM==12) { // INDIR da Master (!DA) al fine di leggere (RW) dati da Slave
					SSPBUF=PARAM[ind_out++];	// preleva da DATOIN e AZZERA BF
					ind_out=ind_out % 4;
					CKP=1; 					// Rilascia il clock
					} else if(COM==44) { // da Master una richiesta di lettura (RW) di un  DAT (DA)
							SSPBUF=PARAM[ind_out++];;	// preleva da DATOIN e AZZERA BF
							ind_out=ind_out % 4;
							CKP=1; 		// rilascia il clock
							} else if (COM==32)	{
									// Fine comunicazioni NACK da Master
									}
	SSPIF=0;
	}				
}

//===================== INIZIALIZZAZIONI ==========================

	
void init_OSC() { 
OSCCON=0B01110000; // 8 Mhz INTIO
}
void init_PORT(void){ 
	// ANSEL= 0Bxx11 1111 RA0..RA4 = AN0 .. AN4 and RB6=AN5
	// SONO i PIN di conversione di 5 temperature AN0..AN4 e VBATTERIA RB6
	ANSEL=0B00111111; 
	/* ================ PORTA ==========================*/
	TRISA=0x00;	PORTA=0x00; TRISA=0B00000000;  
	/* ================ PORTB ========================= */
	TRISB=0x00;	PORTB=0x00; TRISB=0B00000000; 
}

void I2cInitSlave(unsigned char address) 
	{ 
	TRISB1 = 1;		// Impostazione linee SCL/SDA come input
	TRISB4 = 1;		// RB1 = SDA - RB4=SCL
	// SSPCON=xx1x xxxx -> I2c Enable ; xxx1 xxxx= Clock enable; 
	// xxxx 0110 -> 7 bit slave OR xxxx 1110 -> 7 bit slave con interr stop start
	SSPCON = 0B00110110;
	SSPADD=address<<1;// Indirizzo della periferica: bits A7-A1	
	//SSPSTAT=0xxx xxxx;	 0 = 400 Khz  1 = 100 Khz
	SSPSTAT= 0B00000000;
	}
	
void I2cInterrupt(){
	INTCON=0;
	SSPIF=0;
	// Attivazione interrupt I2C
	SSPIE=1;
	PEIE=1;
	GIE=1;
}
void main (void) {
	init_OSC();	 // Settaggio Oscillatore primario
	init_PORT(); // inizializzazione Porte
	I2cInitSlave(ADDR_PIC); // Slave NON include file di Librerie I2c
	I2cInterrupt();
while(1);
}
