/*
Modulo:			i2c.h
Descrizione:	Header file per funzioni i2c
Compilatore:	Hitec-C
Autore:		Bernardo Giovanni - http://www.settorezero.com

Modificato da Giovanni Cupini - giovanni.cupini@tin.it
*/

// Impostare la frequenza del clock del modulo MSSP espressa in KHz
#define	FBUS	400000	// (100khz, 400khz )

// Impostare la frequenza del quarzo espressa in Hertz, _XTAL_FREQ in Main
#define FOSC	_XTAL_FREQ


// Impostare i pin usati per la comunicazione I2C --> MASTER PIC18F4550
#define SCL_PIN	TRISB1
#define SDA_PIN	TRISB0

//===================================================
// Da questa riga in poi non modificare niente
// SSPADD = (FOSC/(4*Clock))-1
#define BAUD (char)((FOSC/(4*FBUS))-1) 
// SLEWRATE 0 per 400KHz, 1 per 100KHz 
#if FBUS == 400000
#define SLEWRATE 0 
#else
#define SLEWRATE 1
#endif

// prototipi funzione
extern void I2cInitMaster (void); // Init MSSP in Master mode
extern void I2cInitSlave (const unsigned char address); // Init MSSP in Slave mode (indirizzo a 7 bit)
extern unsigned char I2cMasterOpenWrite(unsigned char Addr); 
extern unsigned char I2cMasterOpenRead(unsigned char Addr); 
extern void I2cWait(); // Attesa flag interrupt su modulo MSSP
extern void I2cWaitForIdle(); // Attesa bus in stato idle
extern void I2cStart(); // Sequenza di start
extern void I2cRepStart(); // Sequenza di start ripetuto
extern void I2cStop(); // Sequenza di stop
extern unsigned char I2cReadMaster(unsigned char ack); // Lettura in modalit Master
extern unsigned char I2cWriteMaster(unsigned char I2cWriteData); // Scrittura in modalit Master
extern unsigned char I2cReadSlave(); // Lettura in modalit Slave
extern unsigned char I2cGetStrMaster(unsigned  char *Str, unsigned char Lun); // Lettura stringa da Slave
extern unsigned char I2cPutStrMaster(unsigned const char *Str, unsigned char Lun); // Scrittura di una stringa
extern void I2cWriteSlave(unsigned char data); // Scrittura in modalit Slave